package com.digiwin.athena.app.ptc.service.receivables;

import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.ptc.infra.entity.CollectionDetailEntity;
import com.digiwin.athena.app.ptc.infra.entity.ReceivablesDetailEntity;
import com.digiwin.athena.app.ptc.infra.repository.CollectionDetailRepository;
import com.digiwin.athena.app.ptc.infra.repository.ReceivablesDetailRepository;
import com.digiwin.athena.app.ptc.infra.service.CollectionDetailService;
import com.digiwin.athena.app.ptc.infra.service.ReceivablesDetailService;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.generator.SnowflakeWorker;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * @auther: zhenggl
 * @date: 2023/9/12
 */
@Log4j2
@Service
public class ReceivablesInitialCreateEAIService extends AbsEAIService {

    @Resource
    private CollectionDetailRepository collectionDetailRepository;

    @Resource
    private ReceivablesDetailRepository receivablesDetailRepository;


    @Resource
    private ReceivablesDetailService receivablesDetailService;

    @Resource
    private CollectionDetailService collectionDetailService;


    @Override
    public String getServiceName() {
        return ReceivablesUtil.RECEIVABLES_INITIAL_CREATE;
    }



    /** 品号品名规格赋值list*/
    private List<ReceivablesDetailEntity> skuList = Arrays.asList(
            new ReceivablesDetailEntity("24080Y100231","曲股组件（右）","仅前端钻接头孔"),
            new ReceivablesDetailEntity("24070Y100230","直股组件（右）","仅前端钻接头孔"),
            new ReceivablesDetailEntity("24100Y400023","可动心轨辙叉组件（右）","20992mm；无砟道床"),
            new ReceivablesDetailEntity("220500100187","导轨","60kg/m，16193mm（两端均钻接头孔）"),
            new ReceivablesDetailEntity("2111H0320001","60AT1-75单头锻压件（右轨）","U75V，在线淬火轨"),
            new ReceivablesDetailEntity("24020Z100018","曲基本轨-组装（左）","60kg/m，（600+16792+600）mm"),
            new ReceivablesDetailEntity("22040Y200001","曲线尖轨（右）","60AT1，21450mm"),
            new ReceivablesDetailEntity("10940000124","手动扳道器-库改","60AT1，21450mm"),
            new ReceivablesDetailEntity("3011040Y5014","60kg/m钢轨18号可动心轨单开道岔（右）","60-18"),
            new ReceivablesDetailEntity("3011010Y1037","60kg/m钢轨9号单开道岔（右）","60-9（详见：营销2022261号-排20220800083号；顺坡垫板设置2处）（试车线、#64）"),
            new ReceivablesDetailEntity("230300100008","胶接轨","60kg/m，20913（7809+8+13096）mm"),
            new ReceivablesDetailEntity("220600200004","侧向护轨","50kg/m，4800mm"),
            new ReceivablesDetailEntity("241200100015","叉心组件","直股后加长1800mm，不钻接头孔；曲股不加长，钻接头孔"),
            new ReceivablesDetailEntity("2112H0610001","60AT1-模锻翼轨单头锻压件（左轨）","U71Mn，在线淬火轨"),
            new ReceivablesDetailEntity("230109100004","序号9活动心轨（焊接轨件）","60kg/m+60AT1，15020（10737+4283）mm"),
            new ReceivablesDetailEntity("221700200001","左侧尖轨","50AT1，6450mm（按图纸要求钻孔）")
    );


    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        //生成6个随机且不重复的随机整数
        Random random = new Random();
        HashSet<Integer> set = new HashSet<>();
        int count = 0;
        while (count < 6) {
            int num = random.nextInt(26);
            if (!set.contains(num)) {
                set.add(num);
                count++;
            }
        }
        List<Integer> indexList = new ArrayList<>(set);
        //客户名称随机生成
        String[] abcArray = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"};


        Date date = new Date();
        Date date1 = org.apache.commons.lang.time.DateUtils.addDays(date, -1);
        Date date2 = org.apache.commons.lang.time.DateUtils.addDays(date, -7);

        //随机赋值list
        List<ReceivablesDetailEntity> list = Arrays.asList(
                new ReceivablesDetailEntity("鼎捷"+abcArray[indexList.get(0)]+"半导体有限公司",abcArray[indexList.get(0)]+indexList.get(0),"张三","186****9498",new BigDecimal("10000"),new BigDecimal("135"),new BigDecimal("1350000"),new BigDecimal("20000000"),date,"34.2%"),
                new ReceivablesDetailEntity("鼎捷"+abcArray[indexList.get(0)]+"半导体有限公司",abcArray[indexList.get(0)]+indexList.get(0),"李四","158****4794",new BigDecimal("800"),new BigDecimal("2500"),new BigDecimal("2000000"),new BigDecimal("20000000"),date1,"34.2%"),
                new ReceivablesDetailEntity("鼎捷"+abcArray[indexList.get(0)]+"半导体有限公司",abcArray[indexList.get(0)]+indexList.get(0),"王五","150****9529",new BigDecimal("50000"),new BigDecimal("70"),new BigDecimal("3500000"),new BigDecimal("20000000"),date2,"34.2%"),
                new ReceivablesDetailEntity("鼎新"+abcArray[indexList.get(1)]+"半导体有限公司",abcArray[indexList.get(1)]+indexList.get(1),"赵六","139****0530",new BigDecimal("250000"),new BigDecimal("26"),new BigDecimal("6500000"),new BigDecimal("20000000"),date1,"75%"),
                new ReceivablesDetailEntity("鼎新"+abcArray[indexList.get(1)]+"半导体有限公司",abcArray[indexList.get(1)]+indexList.get(1),"陈七","136****5333",new BigDecimal("2500"),new BigDecimal("3400"),new BigDecimal("8500000"),new BigDecimal("20000000"),date2,"75%"),
                new ReceivablesDetailEntity("鼎新"+abcArray[indexList.get(2)]+"半导体有限公司",abcArray[indexList.get(2)]+indexList.get(2),"刘八","188****9299",new BigDecimal("10000"),new BigDecimal("20"),new BigDecimal("200000"),new BigDecimal("5000000"),date1,"40%"),
                new ReceivablesDetailEntity("鼎华"+abcArray[indexList.get(2)]+"半导体有限公司",abcArray[indexList.get(2)]+indexList.get(2),"韩九","130****3555",new BigDecimal("360"),new BigDecimal("5000"),new BigDecimal("1800000"),new BigDecimal("5000000"),date2,"40%"),
                new ReceivablesDetailEntity("鼎华"+abcArray[indexList.get(3)]+"半导体有限公司",abcArray[indexList.get(3)]+indexList.get(3),"管十一","138****9999",new BigDecimal("2500"),new BigDecimal("800"),new BigDecimal("2000000"),new BigDecimal("10000000"),date1,"50%"),
                new ReceivablesDetailEntity("鼎华"+abcArray[indexList.get(3)]+"半导体有限公司",abcArray[indexList.get(3)]+indexList.get(3),"丁十三","187****6160",new BigDecimal("30000"),new BigDecimal("100"),new BigDecimal("3000000"),new BigDecimal("10000000"),date2,"50%"),
                new ReceivablesDetailEntity("鼎华"+abcArray[indexList.get(4)]+"半导体有限公司",abcArray[indexList.get(4)]+indexList.get(4),"包十四","187****6160",new BigDecimal("400"),new BigDecimal("10000"),new BigDecimal("4000000"),new BigDecimal("15000000"),date1,"80%"),
                new ReceivablesDetailEntity("鼎华"+abcArray[indexList.get(4)]+"半导体有限公司",abcArray[indexList.get(4)]+indexList.get(4),"董十五","187****6160",new BigDecimal("100"),new BigDecimal("80000"),new BigDecimal("8000000"),new BigDecimal("15000000"),date2,"80%"),
                new ReceivablesDetailEntity("鼎华"+abcArray[indexList.get(5)]+"半导体有限公司",abcArray[indexList.get(5)]+indexList.get(5),"肖十六","187****6160",new BigDecimal("3500"),new BigDecimal("200"),new BigDecimal("7000000"),new BigDecimal("10000000"),date1,"80%"),
                new ReceivablesDetailEntity("鼎华"+abcArray[indexList.get(5)]+"半导体有限公司",abcArray[indexList.get(5)]+indexList.get(5),"肖十六","187****6160",new BigDecimal("4000"),new BigDecimal("250"),new BigDecimal("1000000"),new BigDecimal("10000000"),date2,"80%"));


        Long tenantSid = SecurityUtil.getUserProfile().getTenantSid();

        //取出已完成的数据
        LambdaQueryWrapper<CollectionDetailEntity> lmq = new LambdaQueryWrapper<>();
        lmq.eq(CollectionDetailEntity::getTenantSid,tenantSid);
        lmq.eq(CollectionDetailEntity::getStatus, "2");
        List<CollectionDetailEntity> collectionDetailEntities = collectionDetailService.list(lmq);

        //删除应收数据
        if (CollectionUtils.isNotEmpty(collectionDetailEntities)){
            receivablesDetailRepository.deleteBatch(collectionDetailEntities,tenantSid);
        }




        int no = 1;
        //随机新增
        for (ReceivablesDetailEntity receivablesDetailEntity : list) {

            int i = random.nextInt(16);
            ReceivablesDetailEntity sku = skuList.get(i);
            receivablesDetailEntity.setSkuCode(sku.getSkuCode());
            receivablesDetailEntity.setSkuName(sku.getSkuName());
            receivablesDetailEntity.setSkuSpec(sku.getSkuSpec());
            receivablesDetailEntity.setReceivableNo("RE-"+SnowflakeWorker.nextId());
            receivablesDetailEntity.setReceivableNon("SEQ-000"+no);
            receivablesDetailEntity.setContractNo("CO"+ SnowflakeWorker.nextId());
            receivablesDetailEntity.setStatus("1");
            receivablesDetailEntity.setSalesOrder("SO-"+SnowflakeWorker.nextId());
            receivablesDetailEntity.setSalesOrderNumber("SON-000"+no);
            receivablesDetailEntity.setSalesmanAssistant("Sp0001");
            receivablesDetailEntity.setSalesmanBoss("qcsupplier001");
            receivablesDetailEntity.setSalesman("qcuser001");
            receivablesDetailEntity.setProduceBoss("Sp0002");
            receivablesDetailEntity.setEmail("ath_Sp0001@163.com");
            receivablesDetailEntity.setWorkNo("work-"+SnowflakeWorker.nextId());
            receivablesDetailEntity.setOverdueDays(String.valueOf((new Date().getTime()-receivablesDetailEntity.getReceivableDate().getTime())/86400000L));
            no = no +1;
        }

        receivablesDetailService.saveBatch(list);

        return buildOK();
    }



}
